<?php
/**
 * wbAMP - Accelerated Mobile Pages for Joomla!
 *
 * @author      Yannick Gaultier
 * @copyright   (c) Yannick Gaultier - Weeblr llc - 2017
 * @package     wbAmp
 * @license     http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version     1.9.2.690
 * @date        2017-06-06
 */

// no direct access
defined('_JEXEC') or die;

/**
 * Base class to handle forms being posted with ajax
 *
 */
class WbampClass_Ampsubmit
{

	/**
	 * Storage for request
	 * @var array
	 */
	protected $input = null;

	/**
	 * @var JUser Current user object
	 */
	protected $user = null;

	/**
	 * Storage for response array
	 *
	 * @var array
	 */
	protected $response = array();

	/**
	 * Stores the post action origin
	 *
	 * @var string
	 */
	protected $origin = '';

	/**
	 * Stores the current site origin URL
	 *
	 * @var string
	 */
	protected $siteOrigin = '';

	/**
	 * Stores the __amp_source_origin query var
	 *
	 * @var string
	 */
	protected $ampSourceOrigin = '';

	/**
	 * Stores the current site host
	 *
	 * @var string
	 */
	protected $siteHost = '';

	/**
	 * List of origins from which CORS-controlled
	 * requests are allowed
	 *
	 * NB: current site URL must be added to the list
	 * at runtime
	 *
	 * @var array
	 */
	protected $allowedOrigins = array(
		'*.ampproject.org',
		'*.amp.cloudflare.com'
	);

	/**
	 * List of headers to be sent in responses
	 *
	 * @var array
	 */
	protected $corsHeaders = array(
		'Content-Type' => 'application/json',
		'Access-Control-Allow-Origin' => '',
		'AMP-Access-Control-Allow-Source-Origin' => '',
		'Access-Control-Expose-Headers' => 'AMP-Access-Control-Allow-Source-Origin',
	);

	/**
	 * List of headers to add to the base list of response headers
	 * stored in $corsHeaders
	 *
	 * If those headers needs dynamic values, one can override the
	 * getResponseHeaders() method to build the desired set of headers
	 *
	 * @var array
	 */
	protected $additionalHeaders = array();

	/**
	 * WeeblrampPostCommentHandler constructor.
	 *
	 * @param JInput $input The request input object.
	 * @param JUser  $data POST data
	 */
	public function __construct($input, $user)
	{
		$this->input = $input;
		$this->user = $user;
		$this->response = array(
			'response' => array(
				'status' => 'ok',
				'message' => '',
				'link' => '',
				'debug' => ''
			),
			'headers' => array()
		);
	}

	/**
	 * Dispatcher, process the request fully.
	 *
	 * NB: we do not use the native Joomla ajax json response output mechanism as:
	 * - it does not handle headers
	 * - it forces output data into a data json field
	 */
	public function process()
	{
		try
		{
			$this->checkInvalidMethod()
			     ->validateOrigin()
			     ->execute()
			     ->outputResponse()
			     ->sendResponse();
		}
		catch (Exception $e)
		{
			ShlSystem_Log::error('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Ajax submit error: ' . $e->getMessage());
			$this->sendResponse('HTTP/1.1 500 Server error');
		}
	}

	/**
	 * Only POST is allowed
	 *
	 * @return $this
	 */
	protected function checkInvalidMethod()
	{
		// discourage random calls
		if ('POST' != $this->input->getMethod())
		{
			$this->sendResponse('HTTP/1.1 405 Method Not Allowed');
		}

		return $this;
	}

	/**
	 * Security checks
	 *
	 * @see https://github.com/ampproject/amphtml/blob/master/spec/amp-cors-requests.md
	 *
	 * @return $this
	 */
	protected function validateOrigin()
	{
		// get headers
		foreach ($_SERVER as $key => $value)
		{
			switch (true)
			{
				case ('http_origin' == strtolower($key)):
					$this->origin = $value;
					break;
				case ('http_amp_same_origin' == strtolower($key)):
					$ampSameOrigin = $value;
					break;
			}
		}

		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Received origin: ' . $this->origin);
		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'http_amp_same_origin: ' . ($ampSameOrigin ? $ampSameOrigin : 'n/a'));

		// get AMP source origin var
		$this->ampSourceOrigin = $this->input->getString('__amp_source_origin', '');
		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, '__amp_source_origin: ' . $this->ampSourceOrigin);

		// compute this site Origin
		$siteUrl = JUri::getInstance();

		$this->siteHost = $siteUrl->getHost();
		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'siteHost: ' . $this->siteHost);
		$this->siteOrigin = $siteUrl->toString(
			array(
				'scheme',
				'host'
			)
		);
		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'siteOrigin: ' . $this->siteOrigin);

		// and store it
		array_unshift(
			$this->allowedOrigins,
			$this->siteOrigin
		);
		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Allowed origins: ' . print_r($this->allowedOrigins, true));

		// validate origin: https://github.com/ampproject/amphtml/blob/master/spec/amp-cors-requests.md
		if (empty($this->origin))
		{
			// No Origin header
			if ('true' !== $ampSameOrigin)
			{
				// not allowed
				ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'No origin header and no http_amp_same_origin, denying.');
				$this->sendResponse('HTTP/1.1 403 FORBIDDEN');
			}
		}
		else
		{
			// Origin header is set
			// 1. Validate Origin header
			if (!$this->isAllowedOrigin($this->origin))
			{
				// not allowed
				ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Origin is not in allowed origins, denying.');
				$this->sendResponse('HTTP/1.1 403 FORBIDDEN');
			}
			else
			{
				ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Origin is in allowed origins, OK.');
			}

			// 2. Validate __amp_source_origin query var
			if (
				empty($this->ampSourceOrigin)
				||
				$this->siteOrigin != $this->ampSourceOrigin
			)
			{
				ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'No __amp_source_origin, or __amp_source_origin is not same as this site origin, denying.');
				$this->sendResponse('HTTP/1.1 403 FORBIDDEN ');
			}
			else
			{
				ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, '__amp_source_origin is valid, OK.');
			}
		}

		return $this;
	}

	/**
	 * Finds out if a received origin is amongst the list of allowed origins,
	 * taking into account wildcards
	 *
	 * @param $origin
	 */
	protected function isAllowedOrigin($origin)
	{
		$origin = JString::rtrim($origin, '/');
		foreach ($this->allowedOrigins as $allowedOrigin)
		{
			$allowedOrigin = JString::rtrim($allowedOrigin, '/');
			ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Allowed origin test: ' . $origin . ' vs ' . $allowedOrigin);
			if ($origin == $allowedOrigin)
			{
				// exact match good to go
				return true;
			}

			// maybe a wildcard in definition?
			if (wbStartsWith($allowedOrigin, '*'))
			{
				// then the incoming origin must finish with the allowed origin minus the wildcard
				$notSoWildAllowedOrigin = wbLTrim($allowedOrigin, '*.');
				ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Allowed origin wildcard test: ' . $origin . ' vs ' . $notSoWildAllowedOrigin);
				if (wbEndsWith($origin, $notSoWildAllowedOrigin))
				{
					ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Allowed origin wildcard test: ' . $origin . ' vs ' . $notSoWildAllowedOrigin);

					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Perform whatever processing is required
	 * to handle the POSTed data, normally through handlers.
	 *
	 * @return $this
	 */
	protected function execute()
	{
		// load possible handlers
		$handlerName = $this->input->getCmd('scope');
		if (empty($handlerName))
		{
			$this->sendResponse('HTTP/1.1 403 FORBIDDEN');
		}

		$handlerClassName = 'WbampModelFeature_' . ucfirst($handlerName);
		if (class_exists($handlerClassName))
		{
			$handler = new $handlerClassName;

			// handle the request, built-in support
			$response = $handler->handleAjax(
				$this->response,
				$this->input,
				$this->user
			);

			$this->response = $response;

			if (!empty($response))
			{
				return $this;
			}
		}

		// if not handled internally, trigger event for 3rd-party support
		$eventName = 'onWbAmpAjaxRequest';
		$params = array(
			'scope' => $handlerName,
			'input' => $this->input,
			'user' => $this->user
		);

		$this->response = JDispatcher::getInstance()->trigger(
			$eventName,
			$params
		);

		// this was our last chance, no one knows how to respond to that request
		if (!$this->response)
		{
			$this->sendResponse('HTTP/1.1 404 Not found');
		}

		return $this;
	}

	/**
	 * Utility: output a JSON response, including appropriate AMP
	 * CORS headers
	 *
	 * @return $this
	 */
	protected function outputResponse()
	{
		$status = $this->getResponseStatus();
		if (!empty($status))
		{
			@header($status);
		}

		// build headers, including dynamic values (origin)
		$headers = $this->getResponseHeaders();

		// output CORS header
		foreach ($headers as $name => $value)
		{
			@header($name . ': ' . $value);
		}

		// respond
		echo json_encode(
			wbArrayGet(
				$this->response,
				'response'
			)
		);

		return $this;
	}

	/**
	 * Builds an array of headers (but do not output them)
	 * based on hard-coded values and dynamic ones (origin)
	 */
	protected function getResponseHeaders()
	{
		// build CORS headers
		$headers = $this->corsHeaders;

		// update live values
		$headers['Access-Control-Allow-Origin'] = $this->origin;
		$headers['AMP-Access-Control-Allow-Source-Origin'] = $this->siteOrigin;

		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Response headers: ' . print_r($headers, true));
		ShlSystem_Log::debug('wbamp', '%s::%d: %s', __METHOD__, __LINE__, 'Response additionalHeaders: ' . print_r($this->additionalHeaders, true));

		// merge any additional headers from response built by handler
		$headers = array_merge(
			$headers,
			wbArrayGet(
				$this->response,
				'headers',
				array()
			)
		);

		return $headers;
	}

	/**
	 * Reads the response status from the response object.
	 */
	protected function getResponseStatus()
	{
		return wbArrayGet(
			$this->response,
			'status',
			''
		);
	}

	/**
	 * Actually send the response, outputting an
	 * optional additional header (used for errors)
	 *
	 * @param string $header
	 */
	protected function sendResponse($header = '')
	{
		if (!empty($header))
		{
			header($header);
		}

		exit;
	}
}


