<?php
/**
 * wbAMP - Accelerated Mobile Pages for Joomla!
 *
 * @author      Yannick Gaultier
 * @copyright   (c) Yannick Gaultier - Weeblr llc - 2017
 * @package     wbAmp
 * @license     http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version     1.9.2.690
 * @date        2017-06-06
 */

// no direct access
defined('_JEXEC') or die;

class WbampModelElement_Disqus
{
	const TYPE_JW_DISQUS = 'jw_disqus';

	private $contentRecord      = null;
	private $context            = null;
	private $input              = null;
	private $disqusPluginparams = null;
	private $supportType        = null;

	/**
	 * Builds a record sufficient to display the Disqus code. Includes current page unique id
	 * and AMP Disqus relay iframe full URL.
	 *
	 * @param array               $currentData Array of all display data gathered so far by the renderer
	 * @param WbampModel_Renderer $renderer The renderer itself
	 *
	 * @return array
	 */
	public function getData($currentData, $renderer)
	{
		if (!empty($this->contentRecord))
		{
			$disqusShortname = WbampHelper_Runtime::$params->get('disqus_comments_shortname');

			// build a unique page identifier for current page
			$pageId = $this->getPageId($disqusShortname, $currentData, $renderer);
			$pageUrl = $this->getPageUrl($disqusShortname, $currentData, $renderer);

			$disqusData = array(
				'endpoint' => WbampHelper_Runtime::$params->get('disqus_comments_endpoint'),
				// use same page id as JW Disqus plugin
				'page_id' => $pageId,
				'page_url' => $pageUrl
			);
		}
		else
		{
			$disqusData = array();
		}

		return $disqusData;
	}

	/**
	 * Builds a unique identifier for the current page. For com_content articles,
	 * default to the JW Disqus plugin format.
	 *
	 * @param string $disqusShortname
	 * @param        $currentData
	 * @param        $renderer
	 *
	 * @return string
	 */
	private function getPageId($disqusShortname, $currentData, $renderer)
	{
		switch ($this->supportType)
		{
			case self::TYPE_JW_DISQUS:
				// default: JW Disqus plugin format
				$pageId = substr(md5($disqusShortname), 0, 10) . '_id' . $this->contentRecord->id;
				break;
			default:
				$uri = JURi::getInstance($currentData['canonical']);
				$path = $uri->getPath();
				$bits = explode('/', JString::trim($path, '/'));
				$pageId = empty($bits) ? '/' : array_pop($bits);
				break;
		}

		// filter to override
		$pageId = ShlHook::filter('wbamp_disqus_identifier', $pageId, $this->contentRecord, $currentData);

		return $pageId;
	}

	/**
	 * Retrieves the page canonical URL, so that page title and other
	 * attributes are correct
	 *
	 * @param string $disqusShortname
	 * @param        $currentData
	 * @param        $renderer
	 *
	 * @return string
	 */
	private function getPageUrl($disqusShortname, $currentData, $renderer)
	{
		switch ($this->supportType)
		{
			case self::TYPE_JW_DISQUS:
			default:
				$pageUrl = $currentData['canonical'];
				break;
		}

		// filter to override
		$pageUrl = ShlHook::filter('wbamp_disqus_url', $pageUrl, $this->contentRecord, $currentData);

		return $pageUrl;
	}

	/**
	 * Register some css classes and regular expressions to be used in the content cleanup process
	 * to specifically remove some content inserted by JW Disqus plugin
	 */
	public function registerCleanup()
	{
		switch ($this->supportType)
		{
			case self::TYPE_JW_DISQUS:
				$updatedCssCleanupList = WbampHelper_Runtime::$params->get('cleanup_css_classes') . "\n" . 'jwDisqusArticleCounter';
				$updatedIdsCleanupList = WbampHelper_Runtime::$params->get('cleanup_css_ids');
				$updatedRegExp = WbampHelper_Runtime::$params->get('cleanup_regexp')
					. "\n" . '#<!-- JoomlaWorks "Disqus.*<!-- DISQUS comments counter and anchor link -->#iuUs => ""'
					. "\n" . '#<!-- DISQUS comments.*JoomlaWorks "Disqus Comments \(for Joomla\)" \(v[0-9]\.[0-9]+\.[0-9]+\) ends here -->#iuUs => ""';

				break;
			default:
				$updatedCssCleanupList = WbampHelper_Runtime::$params->get('cleanup_css_classes');
				$updatedIdsCleanupList = WbampHelper_Runtime::$params->get('cleanup_css_ids');
				$updatedRegExp = WbampHelper_Runtime::$params->get('cleanup_regexp');
				break;
		}
		WbampHelper_Runtime::$params->set(
			'cleanup_css_classes',
			ShlHook::filter(
				'wbamp_disqus_cleanup_css_classes',
				$updatedCssCleanupList
			)
		);
		WbampHelper_Runtime::$params->set(
			'cleanup_css_ids',
			ShlHook::filter(
				'wbamp_disqus_cleanup_css_ids',
				$updatedIdsCleanupList
			)
		);
		WbampHelper_Runtime::$params->set(
			'cleanup_regexp',
			ShlHook::filter(
				'wbamp_disqus_cleanup_regexp',
				$updatedRegExp
			)
		);
	}

	/**
	 * Whether to show Disqus comments or not on the current AMP page.
	 *
	 * Based on code from JW Disqus plugin.
	 *
	 * @param JInput $input
	 * @param string $context
	 * @param Object $contentRecord
	 * @param int    $page
	 */
	public function shouldShow($input, $context, $contentRecord, $page)
	{
		$shouldShow = false;

		// Disqus plugin disabled
		if (JPluginHelper::isEnabled('content', 'jw_disqus'))
		{
			$this->supportType = self::TYPE_JW_DISQUS;
		}

		switch ($this->supportType)
		{
			case self::TYPE_JW_DISQUS:
				$shouldShow = $this->shouldShowJwDisqus($input, $context, $contentRecord, $page);
				break;
			default:
				// by default we don't show Disqus. It can be enabled through filtering wbamp_disqus_should_show,
				// but this assumes all required data is also provided through the other Disqus filters
				break;
		}

		// filtering
		$shouldShow = ShlHook::filter('wbamp_disqus_should_show', $shouldShow, $input, $context, $contentRecord, $page);
		if ($shouldShow)
		{
			$this->contentRecord = $contentRecord;
			$this->input = $input;
			$this->context = $context;
		}

		return $shouldShow;
	}

	/**
	 * Specific support for JW Disqus Plugin: check categories and menu items restrictions.
	 *
	 * @param JInput $input
	 * @param string $context
	 * @param Object $contentRecord
	 * @param int    $page
	 *
	 * @return bool
	 */
	private function shouldShowJwDisqus($input, $context, $contentRecord, $page)
	{
		$shouldShow = false;
		if ($context == 'com_content.article')
		{
			$itemCategory = empty($contentRecord->id) ? null : $contentRecord->id;
			if (empty($itemCategory))
			{
				return $shouldShow;
			}

			$articleId = $input->getInt('id');
			if (empty($articleId))
			{
				return $shouldShow;
			}

			// if com_content && view = article && current cat in cat list && menu item in menu item list
			$option = $input->getCmd('option');
			$view = $input->getCmd('view');
			$Itemid = $input->getInt('Itemid', 999999);

			// load category restrictions
			$this->loadDisqusPluginParams();
			$selectedCategories = (array) $this->disqusPluginparams->get('selectedCategories', array());
			$selectedCategories = array_filter($selectedCategories);

			// load menu restrictions
			$selectedMenus = (array) $this->disqusPluginparams->get('selectedMenus', '');
			$selectedMenus = array_filter($selectedMenus);

			if (
				'com_content' == $option
				&&
				'article' == $view
				&&
				(empty($selectedCategories) || in_array($itemCategory, $selectedCategories))
				&&
				(empty($selectedMenus) || in_array($Itemid, $selectedMenus))
			)
			{
				$shouldShow = true;
			}
		}

		return $shouldShow;
	}

	/**
	 * Loads JW Disqus plugin parameters
	 *
	 * @return JRegistry
	 */
	private function loadDisqusPluginParams()
	{
		if (is_null($this->disqusPluginparams))
		{
			$plugin = JPluginHelper::getPlugin('content', 'jw_disqus');
			$this->disqusPluginparams = new JRegistry($plugin->params);
		}

		return $this->disqusPluginparams;
	}
}
